import { ABCWidgetFactory, DocumentRegistry, IDocumentWidget, DocumentWidget } from '@jupyterlab/docregistry';
import { Signal } from '@lumino/signaling';
import { TextRenderConfig } from 'tde-csvviewer';
import { DataGrid, SelectionModel, DataModel } from 'tde-datagrid';
import { Message } from '@lumino/messaging';
import { Widget } from '@lumino/widgets';
import { EditorModel } from './newmodel';
import { CommandRegistry } from '@lumino/commands';
import { GridSearchService } from './searchservice';
import { Litestore } from './litestore';
import { ListField, MapField } from 'tde-datastore';
export declare class DSVEditor extends Widget {
    private _background;
    /**
     * Construct a new CSV viewer.
     */
    constructor(options: DSVEditor.IOptions);
    /**
     * The CSV widget's context.
     */
    get context(): DocumentRegistry.Context;
    /**
     * A promise that resolves when the csv viewer is ready to be revealed.
     */
    get revealed(): Promise<void>;
    /**
     * The delimiter for the file.
     */
    get delimiter(): string;
    set delimiter(value: string);
    /**
     * The style used by the data grid.
     */
    get style(): DataGrid.Style;
    set style(value: DataGrid.Style);
    get coords(): Array<number | null>;
    /**
     * The config used to create text renderer.
     */
    set rendererConfig(rendererConfig: TextRenderConfig);
    /**
     * The search service
     */
    get searchService(): GridSearchService;
    get grid(): DataGrid;
    /**
     * The DataModel used to render the DataGrid
     */
    get dataModel(): EditorModel;
    get litestore(): Litestore;
    get commandSignal(): Signal<this, DSVEditor.Commands>;
    get dirty(): boolean;
    /**
     * Sets the dirty boolean while also toggling the DIRTY_CLASS
     */
    set dirty(dirty: boolean);
    /**
     * Dispose of the resources used by the widget.
     */
    dispose(): void;
    /**
     * Go to line
     */
    goToLine(lineNumber: number): void;
    /**
     * Handle `'activate-request'` messages.
     */
    protected onActivateRequest(msg: Message): void;
    /**
     * Guess the row delimiter if it was not supplied.
     * This will be fooled if a different line delimiter possibility appears in the first row.
     */
    private _guessRowDelimeter;
    /**
     * Counts the occurrences of a substring from a given string
     */
    private _countOccurrences;
    /**
     * Adds the a column header of alphabets to the top of the data (A..Z,AA..ZZ,AAA...)
     * @param colDelimiter The delimiter used to separated columns (commas, tabs, spaces)
     */
    protected _buildColHeader(colDelimiter: string): string;
    /**
     * Create the model for the grid.
     * TODO: is there a reason we can't just do this once in the constructor?
     */
    protected _updateGrid(): void;
    /**
     * Update the renderer for the grid.
     */
    private _updateRenderer;
    /**
     * Called every time the datamodel updates
     * Updates the file and the litestore
     * @param emitter
     * @param args The row, column, value, record update, selection model
     */
    private _onModelSignal;
    /**
     * Serializes and saves the file (default: asynchronous)
     * @param [exiting] - False to save asynchronously
     */
    save(exiting?: boolean): Promise<void>;
    /**
     * Handles all changes to the data model
     * @param emitter
     * @param command
     */
    private _onCommand;
    /**
     * Updates the current transaction with the raw data, header, and changeArgs
     * @param update The modelChanged args for the Datagrid (may be null)
     */
    updateModel(update?: DSVEditor.ModelChangedArgs): void;
    /**
     * Selects a certain cell using the selection model
     * @param row The row being selected
     * @param column The column being selected
     */
    selectSingleCell(row: number, column: number): void;
    protected getSelectedRange(): SelectionModel.Selection;
    protected onAfterAttach(msg: Message): void;
    private _handlePaste;
    private _cancelEditing;
    private _onMouseClick;
    private _onResize;
    private _region;
    private _row;
    private _column;
    private _context;
    private _grid;
    private _searchService;
    private _monitor;
    private _delimiter;
    private _revealed;
    private _baseRenderer;
    private _litestore;
    private _dirty;
    private _commandSignal;
    private _columnHeader;
    private _rowHeader;
}
export declare namespace DSVEditor {
    /**
     * The Grid update args
     */
    type GridState = {
        currentRows: number;
        currentColumns: number;
        nextChange: DataModel.ChangedArgs;
        nextCommand?: DSVEditor.Commands;
    };
    /**
     * The types of commands that can be made to the model.
     */
    type Commands = 'init' | 'insert-rows-above' | 'insert-rows-below' | 'insert-columns-right' | 'insert-columns-left' | 'remove-rows' | 'remove-columns' | 'move-rows' | 'move-columns' | 'clear-cells' | 'clear-rows' | 'clear-columns' | 'cut-cells' | 'copy-cells' | 'paste-cells' | 'undo' | 'redo' | 'save';
    /**
  
     * The arguments emitted to the Editor when the datamodel changes
     */
    type ModelChangedArgs = {
        rowUpdate?: ListField.Update<number>;
        columnUpdate?: ListField.Update<number>;
        valueUpdate?: MapField.Update<string>;
        gridStateUpdate?: GridState;
        type?: string;
        selection?: SelectionModel.Selection;
    };
    const SCHEMA_ID = "datamodel";
    const RECORD_ID = "datamodel";
    const DATAMODEL_SCHEMA: {
        id: string;
        fields: {
            rowMap: ListField<number>;
            columnMap: ListField<number>;
            valueMap: MapField<string>;
            selection: import("tde-datastore").RegisterField<SelectionModel.Selection>;
            gridState: import("tde-datastore").RegisterField<GridState>;
            type: import("tde-datastore").RegisterField<string>;
        };
    };
    const NULL_NUMS: number[];
    const NULL_NUM_SPLICE: {
        index: number;
        remove: number;
        values: number[];
    };
    const NULL_CHANGE: GridState[];
    const NULL_CHANGE_SPLICE: {
        index: number;
        remove: number;
        values: GridState[];
    };
}
export declare class EditableCSVDocumentWidget extends DocumentWidget<DSVEditor> {
    constructor(options: EditableCSVDocumentWidget.IOptions);
    /**
     * Disposes the current widget, handles save dialog
     */
    dispose(): Promise<void>;
    /**
     * Set URI fragment identifier for rows
     */
    setFragment(fragment: string): void;
}
export declare namespace EditableCSVDocumentWidget {
    interface IOptions extends DocumentWidget.IOptionsOptionalContent<DSVEditor> {
        delimiter?: string;
        commandRegistry: CommandRegistry;
    }
}
export declare class EditableCSVViewerFactory extends ABCWidgetFactory<IDocumentWidget<DSVEditor>> {
    constructor(options: DocumentRegistry.IWidgetFactoryOptions<IDocumentWidget>, commandRegistry: CommandRegistry);
    createNewWidget(context: DocumentRegistry.Context): IDocumentWidget<DSVEditor>;
    private _commandReigstry;
}
export declare namespace DSVEditor {
    /**
     * Instantiation options for CSV widgets.
     */
    interface IOptions {
        /**
         * The document context for the CSV being rendered by the widget.
         */
        context: DocumentRegistry.Context;
    }
}
