var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { CommandToolbarButton } from '@jupyterlab/apputils';
import { ActivityMonitor } from '@jupyterlab/coreutils';
import { ABCWidgetFactory, DocumentWidget } from '@jupyterlab/docregistry';
import { PromiseDelegate } from '@lumino/coreutils';
import { Signal } from '@lumino/signaling';
import { BasicKeyHandler, BasicSelectionModel, DataGrid, TextRenderer } from 'tde-datagrid';
import { PanelLayout, Widget } from '@lumino/widgets';
import { EditorModel } from './newmodel';
import { RichMouseHandler } from './handler';
import { numberToCharacter } from './_helper';
import { toArray, range } from '@lumino/algorithm';
import { CommandIDs } from './index';
import { VirtualDOM, h } from '@lumino/virtualdom';
import { GridSearchService } from './searchservice';
import { Litestore } from './litestore';
import { Fields } from 'tde-datastore';
import { unsaveDialog } from './dialog';
const CSV_CLASS = 'jp-CSVViewer';
const CSV_GRID_CLASS = 'jp-CSVViewer-grid';
const COLUMN_HEADER_CLASS = 'jp-column-header';
const ROW_HEADER_CLASS = 'jp-row-header';
const BACKGROUND_CLASS = 'jp-background';
const DIRTY_CLASS = 'jp-mod-dirty';
const RENDER_TIMEOUT = 1000;
export class DSVEditor extends Widget {
    /**
     * Construct a new CSV viewer.
     */
    constructor(options) {
        super();
        this._monitor = null;
        this._delimiter = ',';
        this._revealed = new PromiseDelegate();
        this._baseRenderer = null;
        this._dirty = false;
        // Signals for basic editing functionality
        this._commandSignal = new Signal(this);
        const context = (this._context = options.context);
        const layout = (this.layout = new PanelLayout());
        this.addClass(CSV_CLASS);
        //Datagrid Size
        this._grid = new DataGrid({
            defaultSizes: {
                rowHeight: 24,
                columnWidth: 144,
                rowHeaderWidth: 64,
                columnHeaderHeight: 36
            },
            headerVisibility: 'all'
        });
        this._grid.addClass(CSV_GRID_CLASS);
        this._grid.headerVisibility = 'all';
        this._grid.keyHandler = new BasicKeyHandler();
        this._grid.copyConfig = {
            separator: '\t',
            format: DataGrid.copyFormatGeneric,
            headers: 'none',
            warningThreshold: 1e6
        };
        const handler = new RichMouseHandler({ grid: this._grid });
        this._grid.mouseHandler = handler;
        handler.clickSignal.connect(this._onMouseClick, this);
        handler.resizeSignal.connect(this._onResize, this);
        layout.addWidget(this._grid);
        // init search service to search for matches with the data grid
        this._searchService = new GridSearchService(this._grid);
        this._searchService.changed.connect(this._updateRenderer, this);
        // add the background column and row header elements
        this._background = VirtualDOM.realize(h.div({
            className: BACKGROUND_CLASS,
            style: {
                position: 'absolute',
                zIndex: '1'
            }
        }));
        this._rowHeader = VirtualDOM.realize(h.div({
            className: ROW_HEADER_CLASS,
            style: {
                position: 'absolute',
                zIndex: '2'
            }
        }));
        this._columnHeader = VirtualDOM.realize(h.div({
            className: COLUMN_HEADER_CLASS,
            style: {
                position: 'absolute',
                zIndex: '2'
            }
        }));
        // append the column and row headers to the viewport
        this._grid.viewport.node.appendChild(this._rowHeader);
        this._grid.viewport.node.appendChild(this._columnHeader);
        this._grid.viewport.node.appendChild(this._background);
        void this._context.ready.then(() => {
            this._updateGrid();
            this._revealed.resolve(undefined);
            // Throttle the rendering rate of the widget.
            this._monitor = new ActivityMonitor({
                signal: context.model.contentChanged,
                timeout: RENDER_TIMEOUT
            });
            this._monitor.activityStopped.connect(this._updateGrid, this);
        });
        this._grid.editingEnabled = true;
        this.commandSignal.connect(this._onCommand, this);
    }
    /**
     * The CSV widget's context.
     */
    get context() {
        return this._context;
    }
    /**
     * A promise that resolves when the csv viewer is ready to be revealed.
     */
    get revealed() {
        return this._revealed.promise;
    }
    /**
     * The delimiter for the file.
     */
    get delimiter() {
        return this._delimiter;
    }
    set delimiter(value) {
        if (value === this._delimiter) {
            return;
        }
        this._delimiter = value;
        this._updateGrid();
    }
    /**
     * The style used by the data grid.
     */
    get style() {
        return this._grid.style;
    }
    set style(value) {
        this._grid.style = value;
    }
    get coords() {
        return [this._row, this._column];
    }
    /**
     * The config used to create text renderer.
     */
    set rendererConfig(rendererConfig) {
        this._baseRenderer = rendererConfig;
        this._updateRenderer();
    }
    /**
     * The search service
     */
    get searchService() {
        return this._searchService;
    }
    get grid() {
        return this._grid;
    }
    /**
     * The DataModel used to render the DataGrid
     */
    get dataModel() {
        return this._grid.dataModel;
    }
    get litestore() {
        return this._litestore;
    }
    get commandSignal() {
        return this._commandSignal;
    }
    get dirty() {
        return this._dirty;
    }
    /**
     * Sets the dirty boolean while also toggling the DIRTY_CLASS
     */
    set dirty(dirty) {
        this._dirty = dirty;
        if (this.dirty && !this.title.className.includes(DIRTY_CLASS)) {
            this.title.className += DIRTY_CLASS;
        }
        else if (!this.dirty) {
            this.title.className = this.title.className.replace(DIRTY_CLASS, '');
        }
    }
    /**
     * Dispose of the resources used by the widget.
     */
    dispose() {
        if (this._monitor) {
            this._monitor.dispose();
        }
        super.dispose();
    }
    /**
     * Go to line
     */
    goToLine(lineNumber) {
        this._grid.scrollToRow(lineNumber);
    }
    /**
     * Handle `'activate-request'` messages.
     */
    onActivateRequest(msg) {
        this.node.tabIndex = -1;
        this.node.focus();
    }
    /**
     * Guess the row delimiter if it was not supplied.
     * This will be fooled if a different line delimiter possibility appears in the first row.
     */
    _guessRowDelimeter(data) {
        const i = data.slice(0, 5000).indexOf('\r');
        if (i === -1) {
            return '\n';
        }
        else if (data[i + 1] === '\n') {
            return '\r\n';
        }
        else {
            return '\r';
        }
    }
    /**
     * Counts the occurrences of a substring from a given string
     */
    _countOccurrences(string, substring, rowDelimiter) {
        let numCol = 0;
        let pos = 0;
        const l = substring.length;
        const firstRow = string.slice(0, string.indexOf(rowDelimiter));
        pos = firstRow.indexOf(substring, pos);
        while (pos !== -1) {
            numCol++;
            pos += l;
            pos = firstRow.indexOf(substring, pos);
        }
        // number of columns is the amount of columns + 1
        return numCol + 1;
    }
    /**
     * Adds the a column header of alphabets to the top of the data (A..Z,AA..ZZ,AAA...)
     * @param colDelimiter The delimiter used to separated columns (commas, tabs, spaces)
     */
    _buildColHeader(colDelimiter) {
        const rawData = this._context.model.toString();
        const alphabet = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
        // when the model is first created, we don't know how many columns or what the row delimeter is
        const rowDelimiter = this._guessRowDelimeter(rawData);
        const numCol = this._countOccurrences(rawData, colDelimiter, rowDelimiter);
        // if only single alphabets fix the string
        if (numCol <= 26) {
            return (alphabet
                .slice(0, numCol)
                .split('')
                .join(colDelimiter) + rowDelimiter);
        }
        // otherwise compute the column header with multi-letters (AA..)
        else {
            // get all single letters
            let columnHeader = alphabet.split('').join(colDelimiter);
            // find the rest
            for (let i = 27; i < numCol; i++) {
                columnHeader += colDelimiter + numberToCharacter(i);
            }
            return columnHeader + rowDelimiter;
        }
    }
    /**
     * Create the model for the grid.
     * TODO: is there a reason we can't just do this once in the constructor?
     */
    _updateGrid() {
        const delimiter = this.delimiter;
        const model = this._grid.dataModel;
        let data;
        if (!model) {
            data = this._context.model.toString();
            const dataModel = (this._grid.dataModel = new EditorModel({
                data,
                delimiter
            }));
            this._grid.selectionModel = new BasicSelectionModel({ dataModel });
            // create litestore
            this._litestore = new Litestore({
                id: 0,
                schemas: [DSVEditor.DATAMODEL_SCHEMA]
            });
            // Give the litestore as a property of the model for it to read from.
            this.dataModel.litestore = this._litestore;
            // Define the initial update object for the litestore.
            const update = {};
            // Define the initial state of the row and column map.
            const rowUpdate = {
                index: 0,
                remove: 0,
                values: toArray(range(0, this.dataModel.totalRows()))
            };
            const columnUpdate = {
                index: 0,
                remove: 0,
                values: toArray(range(0, this.dataModel.totalColumns()))
            };
            // Add the map updates to the update object.
            update.rowUpdate = rowUpdate;
            update.columnUpdate = columnUpdate;
            // set inital status of litestore
            this.updateModel(update);
            dataModel.onChangedSignal.connect(this._onModelSignal, this);
            // dataModel.cancelEditingSignal.connect(this._cancelEditing, this);
        }
        // update the position of the background row and column headers
        this._background.style.width = `${this._grid.bodyWidth}px`;
        this._background.style.height = `${this._grid.bodyHeight}px`;
        this._background.style.left = `${this._grid.headerWidth}px`;
        this._background.style.top = `${this._grid.headerHeight}px`;
        this._columnHeader.style.left = `${this._grid.headerWidth}px`;
        this._columnHeader.style.height = `${this._grid.headerHeight}px`;
        this._columnHeader.style.width = `${this._grid.bodyWidth}px`;
        this._rowHeader.style.top = `${this._grid.headerHeight}px`;
        this._rowHeader.style.width = `${this._grid.headerWidth}px`;
        this._rowHeader.style.height = `${this._grid.bodyHeight}px`;
    }
    /**
     * Update the renderer for the grid.
     */
    _updateRenderer() {
        if (this._baseRenderer === null) {
            return;
        }
        const rendererConfig = this._baseRenderer;
        const renderer = new TextRenderer({
            textColor: rendererConfig.textColor,
            horizontalAlignment: rendererConfig.horizontalAlignment,
            backgroundColor: this._searchService.cellBackgroundColorRendererFunc(rendererConfig)
        });
        this._grid.cellRenderers.update({
            body: renderer,
            'column-header': renderer,
            'corner-header': renderer,
            'row-header': renderer
        });
    }
    /**
     * Called every time the datamodel updates
     * Updates the file and the litestore
     * @param emitter
     * @param args The row, column, value, record update, selection model
     */
    _onModelSignal(emitter, args) {
        this.updateModel(args);
    }
    /**
     * Serializes and saves the file (default: asynchronous)
     * @param [exiting] - False to save asynchronously
     */
    async save(exiting = false) {
        const newString = this.dataModel.updateString();
        this.context.model.fromString(newString);
        exiting ? await this.context.save() : this.context.save();
        // reset boolean since no new changes exist
        this.dirty = false;
    }
    // private _cancelEditing(emitter: EditorModel): void {
    //   this._grid.editorController.cancel();
    // }
    /**
     * Handles all changes to the data model
     * @param emitter
     * @param command
     */
    _onCommand(emitter, command) {
        const selectionModel = this._grid.selectionModel;
        const selection = selectionModel.currentSelection();
        let r1, r2, c1, c2;
        // grab selection if it exists
        if (selection) {
            r1 = Math.min(selection.r1, selection.r2);
            r2 = Math.max(selection.r1, selection.r2);
            c1 = Math.min(selection.c1, selection.c2);
            c2 = Math.max(selection.c1, selection.c2);
        }
        const newSelection = {
            r1,
            r2,
            c1,
            c2,
            cursorColumn: c1,
            cursorRow: r1,
            clear: 'all'
        };
        // Set up the update object for the litestore.
        let update = null;
        switch (command) {
            case 'insert-rows-above': {
                update = this.dataModel.addRows(this._region, this._row);
                // Add the type property so we can differentiate an insert above from an insert below.
                update.type = command;
                break;
            }
            case 'insert-rows-below': {
                update = this.dataModel.addRows(this._region, this._row + 1);
                // Add the command to the grid state.
                update.gridStateUpdate.nextCommand = command;
                // move the selection down a row to account for the new row being inserted
                newSelection.r1 += 1;
                newSelection.r2 += 1;
                break;
            }
            case 'insert-columns-left': {
                update = this.dataModel.addColumns(this._region, this._column);
                break;
            }
            case 'insert-columns-right': {
                update = this.dataModel.addColumns(this._region, this._column + 1);
                // move the selection right a column to account for the new column being inserted
                newSelection.c1 += 1;
                newSelection.c2 += 1;
                break;
            }
            case 'remove-rows': {
                update = this.dataModel.removeRows(this._region, this._row);
                break;
            }
            case 'remove-columns': {
                update = this.dataModel.removeColumns(this._region, this._column);
                break;
            }
            case 'cut-cells':
                // Copy to the OS clipboard.
                this._grid.copyToClipboard();
                // Cut the cell selection.
                update = this.dataModel.cut('body', r1, c1, r2, c2);
                break;
            case 'copy-cells': {
                // Copy to the OS clipboard.
                this._grid.copyToClipboard();
                // Make a local copy of the cells.
                this.dataModel.copy('body', r1, c1, r2, c2);
                break;
            }
            case 'paste-cells': {
                // Paste the cells in the region.
                update = this.dataModel.paste('body', r1, c1);
                // By default, upper left cell get's re-edited, so we need to cancel.
                this._cancelEditing();
                break;
            }
            case 'clear-cells': {
                update = this.dataModel.clearCells(this._region, { r1, r2, c1, c2 });
                break;
            }
            case 'clear-rows': {
                const rowSpan = Math.abs(r1 - r2) + 1;
                update = this.dataModel.clearRows(this._region, r1, rowSpan);
                break;
            }
            case 'clear-columns': {
                const columnSpan = Math.abs(c1 - c2) + 1;
                update = this.dataModel.clearColumns(this._region, c1, columnSpan);
                break;
            }
            case 'undo': {
                // check to see if an undo exists (one undo will exist because that's the initial transaction)
                if (this._litestore.transactionStore.undoStack.length === 1) {
                    return;
                }
                const { gridState, selection } = this._litestore.getRecord({
                    schema: DSVEditor.DATAMODEL_SCHEMA,
                    record: DSVEditor.RECORD_ID
                });
                this._litestore.undo();
                // Have the model emit the opposite change to the Grid.
                this.dataModel.emitOppositeChange(gridState.nextChange);
                if (!selection) {
                    break;
                }
                // reselect the previous selection.
                const { r1, r2, c1, c2 } = selection;
                this._grid.selectionModel.select({
                    r1,
                    r2,
                    c1,
                    c2,
                    cursorRow: r1,
                    cursorColumn: c1,
                    clear: 'all'
                });
                break;
            }
            case 'redo': {
                // check to see if an redo exists (one redo will exist because that's the initial transaction)
                if (this._litestore.transactionStore.redoStack.length === 0) {
                    return;
                }
                // Redo first, then get the new selection and the new grid change.
                this._litestore.redo();
                const { gridState, selection } = this._litestore.getRecord({
                    schema: DSVEditor.DATAMODEL_SCHEMA,
                    record: DSVEditor.RECORD_ID
                });
                // Have the data model emit the grid change to the grid.
                this.dataModel.emitCurrentChange(gridState.nextChange);
                if (!selection) {
                    break;
                }
                const command = gridState.nextCommand;
                const gridChange = gridState.nextChange;
                let { r1, r2, c1, c2 } = selection;
                let move;
                // handle special cases for selection
                if (command === 'insert-rows-below') {
                    r1 += 1;
                    r2 += 1;
                }
                else if (command === 'insert-columns-right') {
                    c1 += 1;
                    c2 += 1;
                }
                else if (command === 'move-rows') {
                    move = gridChange;
                    r1 = move.destination;
                    r2 = move.destination;
                }
                else if (command === 'move-columns') {
                    move = gridChange;
                    c1 = move.destination;
                    c2 = move.destination;
                }
                // Make the new selection.
                this._grid.selectionModel.select({
                    r1,
                    r2,
                    c1,
                    c2,
                    cursorRow: r1,
                    cursorColumn: c1,
                    clear: 'all'
                });
                break;
            }
            case 'save':
                this.save();
                break;
        }
        if (update) {
            update.selection = selection;
            // Add the command to the grid state.
            update.gridStateUpdate.nextCommand = command;
            this.updateModel(update);
            this._grid.selectionModel.select(newSelection);
        }
    }
    /**
     * Updates the current transaction with the raw data, header, and changeArgs
     * @param update The modelChanged args for the Datagrid (may be null)
     */
    updateModel(update) {
        // grab current selection if none exists
        if (!update.selection) {
            update.selection = this._grid.selectionModel.currentSelection();
        }
        // for every litestore change except the init, set the dirty boolean to true
        this.dirty =
            update &&
                update.gridStateUpdate &&
                update.gridStateUpdate.nextCommand === 'init'
                ? false
                : true;
        // Update the litestore.
        this._litestore.beginTransaction();
        this._litestore.updateRecord({
            schema: DSVEditor.DATAMODEL_SCHEMA,
            record: DSVEditor.RECORD_ID
        }, {
            rowMap: update.rowUpdate || DSVEditor.NULL_NUM_SPLICE,
            columnMap: update.columnUpdate || DSVEditor.NULL_NUM_SPLICE,
            valueMap: update.valueUpdate || null,
            selection: update.selection || null,
            gridState: update.gridStateUpdate || null
        });
        this._litestore.endTransaction();
    }
    /**
     * Selects a certain cell using the selection model
     * @param row The row being selected
     * @param column The column being selected
     */
    selectSingleCell(row, column) {
        const select = {
            r1: row,
            r2: row,
            c1: column,
            c2: column,
            cursorRow: row,
            cursorColumn: column,
            clear: 'all'
        };
        this._grid.selectionModel.select(select);
    }
    getSelectedRange() {
        const selections = toArray(this._grid.selectionModel.selections());
        if (selections.length === 0) {
            return;
        }
        return selections[0];
    }
    onAfterAttach(msg) {
        super.onAfterAttach(msg);
        this.node.addEventListener('paste', this._handlePaste.bind(this));
    }
    _handlePaste(event) {
        const copiedText = event.clipboardData.getData('text/plain');
        // prevent default behavior
        event.preventDefault();
        event.stopPropagation();
        const { r1, r2, c1, c2 } = this.getSelectedRange();
        const row = Math.min(r1, r2);
        const column = Math.min(c1, c2);
        const update = this.dataModel.paste(this._region, row, column, copiedText);
        this._cancelEditing();
        this.updateModel(update);
    }
    _cancelEditing() {
        this._grid.editorController.cancel();
    }
    _onMouseClick(emitter, hit) {
        if (hit.region !== 'void') {
            this._region = hit.region;
        }
        this._row = hit.row;
        this._column = hit.column;
    }
    _onResize(emitter) {
        this._background.style.width = `${this._grid.bodyWidth}px`;
        this._background.style.height = `${this._grid.bodyHeight}px`;
        this._background.style.left = `${this._grid.headerWidth}px`;
        this._background.style.top = `${this._grid.headerHeight}px`;
        this._columnHeader.style.left = `${this._grid.headerWidth}px`;
        this._columnHeader.style.height = `${this._grid.headerHeight}px`;
        this._columnHeader.style.width = `${this._grid.bodyWidth}px`;
        this._rowHeader.style.top = `${this._grid.headerHeight}px`;
        this._rowHeader.style.width = `${this._grid.headerWidth}px`;
        this._rowHeader.style.height = `${this._grid.bodyHeight}px`;
    }
}
(function (DSVEditor) {
    DSVEditor.SCHEMA_ID = 'datamodel';
    DSVEditor.RECORD_ID = 'datamodel';
    DSVEditor.DATAMODEL_SCHEMA = {
        id: DSVEditor.SCHEMA_ID,
        fields: {
            rowMap: Fields.List(),
            columnMap: Fields.List(),
            valueMap: Fields.Map(),
            selection: Fields.Register({
                value: null
            }),
            gridState: Fields.Register({
                value: null
            }),
            type: Fields.String()
        }
    };
    DSVEditor.NULL_NUMS = [];
    DSVEditor.NULL_NUM_SPLICE = { index: 0, remove: 0, values: DSVEditor.NULL_NUMS };
    DSVEditor.NULL_CHANGE = [];
    DSVEditor.NULL_CHANGE_SPLICE = {
        index: 0,
        remove: 0,
        values: DSVEditor.NULL_CHANGE
    };
})(DSVEditor || (DSVEditor = {}));
export class EditableCSVDocumentWidget extends DocumentWidget {
    constructor(options) {
        let { content, reveal } = options;
        const { context, commandRegistry } = options, other = __rest(options, ["context", "commandRegistry"]);
        content = content || new DSVEditor({ context });
        reveal = Promise.all([reveal, content.revealed]);
        super(Object.assign({ context, content, reveal }, other));
        // add commands to the toolbar
        const commands = commandRegistry;
        const { save, undo, redo, cutToolbar, copyToolbar, pasteToolbar } = CommandIDs;
        this.toolbar.addItem('save', new CommandToolbarButton({ commands, id: save }));
        this.toolbar.addItem('undo', new CommandToolbarButton({ commands, id: undo }));
        this.toolbar.addItem('redo', new CommandToolbarButton({ commands, id: redo }));
        this.toolbar.addItem('cut', new CommandToolbarButton({ commands, id: cutToolbar }));
        this.toolbar.addItem('copy', new CommandToolbarButton({ commands, id: copyToolbar }));
        this.toolbar.addItem('paste', new CommandToolbarButton({ commands, id: pasteToolbar }));
        /* possible feature
        const filterData = new FilterButton({ selected: content.delimiter });
        this.toolbar.addItem('filter-data', filterData);
        */
    }
    /**
     * Disposes the current widget, handles save dialog
     */
    async dispose() {
        // if there are unsaved changes, prompt dialog
        if (this.content.dirty && !this.isDisposed) {
            const dialog = unsaveDialog(this.content);
            const result = await dialog.launch();
            dialog.dispose();
            // on Cancel, remove dialog
            if (result.button.label === 'Cancel') {
                return;
            }
            // on Save, save the file
            if (result.button.label === 'Save') {
                await this.content.save(true);
            }
        }
        super.dispose();
    }
    /**
     * Set URI fragment identifier for rows
     */
    setFragment(fragment) {
        const parseFragments = fragment.split('=');
        // TODO: expand to allow columns and cells to be selected
        // reference: https://tools.ietf.org/html/rfc7111#section-3
        if (parseFragments[0] !== '#row') {
            return;
        }
        // multiple rows, separated by semi-colons can be provided, we will just
        // go to the top one
        let topRow = parseFragments[1].split(';')[0];
        // a range of rows can be provided, we will take the first value
        topRow = topRow.split('-')[0];
        // go to that row
        void this.context.ready.then(() => {
            this.content.goToLine(Number(topRow));
        });
    }
}
export class EditableCSVViewerFactory extends ABCWidgetFactory {
    constructor(options, commandRegistry) {
        super(options);
        this._commandReigstry = commandRegistry;
    }
    createNewWidget(context) {
        const commandRegistry = this._commandReigstry;
        return new EditableCSVDocumentWidget({ context, commandRegistry });
    }
}
//# sourceMappingURL=widget.js.map